<?php

defined( 'ABSPATH' ) || exit;

class Radio_Player_REST_API extends WP_REST_Controller {
	/** @var null */
	private static $instance = null;

	/**
	 * Radio_Player_REST_API constructor.
	 */
	public function __construct() {
		$this->namespace = 'radio-player/v1';

		add_action( 'rest_api_init', [ $this, 'register_api' ] );
	}

	/**
	 * Register REST API routes
	 *
	 * @since 1.0.0
	 */
	public function register_api() {

		//get all players
		register_rest_route( $this->namespace, '/player/', array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'get_player' ),
				'permission_callback' => '__return_true',
			),
		) );

		// Update Player Data
		register_rest_route( $this->namespace, '/update-player/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'update_player' ),
				'permission_callback' => array( $this, 'check_permission' )
			),
		) );

		// Update settings
		register_rest_route( $this->namespace, '/settings/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'update_settings' ),
				'permission_callback' => array( $this, 'check_permission' ),
			),
		) );

		//delete player
		register_rest_route( $this->namespace, '/delete-player/', array(
			array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => array( $this, 'delete_player' ),
				'permission_callback' => array( $this, 'check_delete_permission' ),
			),
		) );

		// Handle notice
		register_rest_route( $this->namespace, '/handle-notice/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'handle_notice' ),
				'permission_callback' => '__return_true',
			),
		) );

		// Duplicate Player
		register_rest_route( $this->namespace, '/duplicate-player/', array(
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'duplicate_player' ),
				'permission_callback' => '__return_true',
			),
		) );

	}

	public function duplicate_player( $request ) {
		$ids = $request->get_param( 'ids' );

		$data = [];
		if ( ! empty( $ids ) ) {

			foreach ( $ids as $id ) {
				$player = radio_player_get_player( $id );

				if ( $player ) {
					$player['title']      = 'Copy of ' . $player['title'];
					$player['created_at'] = current_time( 'mysql' );
					$player['updated_at'] = current_time( 'mysql' );
					$player['locations']  = serialize( [] );

					$insert_id = $this->update_player( $player );

					$data = array_merge( $player, [
						'id'     => $insert_id,
						'config' => unserialize( $player['config'] ),
					] );

					return $data;
				}
			}
		}

		return rest_ensure_response( $data );
	}

	public function handle_notice( $request ) {
		$data = json_decode( $request->get_body() );

		$type  = $data->type;
		$value = $data->value;

		if ( 'rating' == $type ) {
			if ( 'hide_notice' == $value ) {
				update_option( 'radio_player_rating_notice', 'off' );
			} else {
				set_transient( 'radio_player_rating_notice_interval', 'off', $value * DAY_IN_SECONDS );
			}
		}

		update_option( sanitize_key( 'radio_player_notices' ), [] );

		wp_send_json_success( true );
	}

	public function delete_player( $data ) {
		$player_id = isset( $data['id'] ) ? $data['id'] : '';

		global $wpdb;

		$wpdb->delete( $wpdb->prefix . 'radio_player_players', [ 'id' => $player_id ] );

		return rest_ensure_response( true );
	}

	public function update_settings( $request ) {
		$data = json_decode( $request->get_body(), 1 );

		update_option( 'radio_player_settings', $data );

		return rest_ensure_response( true );
	}

	/**
	 * @return bool
	 */
	public function check_permission() {
		return current_user_can( 'manage_options' );
	}

	public function check_delete_permission() {
		return current_user_can( 'delete_others_posts' );
	}


	/**
	 * Update player data
	 *
	 * @param $request
	 */
	public function update_player( $request ) {
		global $wpdb;
		$table = $wpdb->prefix . 'radio_player_players';

		$posted = is_object( $request ) ? $request->get_json_params() : $request;

		$id = ! empty( $posted['id'] ) ? $posted['id'] : '';

		$data = [
			'id'     => $id,
			'title'  => ! empty( $posted['title'] ) ? $posted['title'] : '',
			'status' => ! empty( $posted['status'] ) ? 1 : 0,
			'config' => serialize( $posted ),
		];

		if ( $id && is_object( $request ) ) {
			$wpdb->update( $table, $data, [ 'id' => $id ] );
		} else {
			$wpdb->insert( $table, $data );
		}

		$player_id = $wpdb->insert_id ? $wpdb->insert_id : $id;

		$data['id']     = $player_id;
		$data['config'] = $posted;

		if ( ! is_object( $request ) ) {
			return $data;
		}

		return rest_ensure_response( $data );
	}


	/**
	 * Get all players callback
	 *
	 * @since 1.0.0
	 */
	public function get_player() {
		$players = radio_player_get_player();

		return rest_ensure_response( $players );
	}

	/**
	 * @return Radio_Player_REST_API|null
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}

Radio_Player_REST_API::instance();

